@@ -20,10 +20,11 @@
 
 import os
 import gobject
-from lib import thetvdbapi, serializer
+from lib import thetvdbapi, serializer, constants
+from lib.util import get_color, image_downloader
 from xml.etree import ElementTree as ET
 from asyncworker import AsyncWorker, AsyncItem
-from lib.constants import TVDB_API_KEY
+from lib.constants import TVDB_API_KEY, DATA_DIR
 from datetime import datetime
 import gettext
 
@@ -32,7 +33,8 @@ _ = gettext.gettext
 class Show(object):
     
     def __init__(self, name, genre = None, overview = None, network = None,
-                 rating = None, actors = [], episode_list = [], thetvdb_id = -1):
+                 rating = None, actors = [], episode_list = [], image = None,
+                 thetvdb_id = -1):
         self.id = -1
         self.name = name
         self.genre = genre
@@ -41,6 +43,8 @@ class Show(object):
         self.rating = rating
         self.actors = actors
         self.episode_list = episode_list
+        self.image = image
+        self.season_images = {}
         self.thetvdb_id = thetvdb_id
     
     def get_episodes_by_season(self, season_number):
@@ -103,6 +107,82 @@ class Show(object):
     def __str__(self):
         return self.name
 
+    def get_info_markup(self):
+        seasons = len(self.get_seasons())
+        if seasons:
+            color = get_color(constants.SECONDARY_TEXT_COLOR)
+            episodes_info = self.get_episodes_info()
+            episodes_to_watch = episodes_info['episodes_to_watch']
+            next_episode = episodes_info['next_episode']
+            if next_episode and next_episode.already_aired():
+                color = get_color(constants.ACTIVE_TEXT_COLOR) 
+            show_info = '\n<small><span foreground="%s">' % color
+            show_info += gettext.ngettext('%s season', '%s seasons', seasons) \
+                         % seasons
+            if self.is_completely_watched():
+                show_info += ' | ' + _('Completely watched')
+            else:
+                if episodes_to_watch:
+                    n_episodes_to_watch = len(episodes_to_watch)
+                    show_info += ' | ' + gettext.ngettext('%s episode not watched',
+                                                          '%s episodes not watched',
+                                                          n_episodes_to_watch) \
+                                                          % n_episodes_to_watch
+                    if next_episode:
+                        next_air_date = next_episode.air_date
+                        if next_air_date:
+                            show_info += ' | ' + _('<i>Next air date:</i> ep. %s on %s') % \
+                                         (next_episode.get_episode_show_number(), \
+                                         next_episode.get_air_date_text())
+                        else:
+                            show_info += ' | ' + _('<i>Next to watch:</i> %s') % \
+                                         next_episode
+                        if next_episode.already_aired():
+                            color = get_color(constants.ACTIVE_TEXT_COLOR)
+                else:
+                    show_info += ' | ' + _('No episodes to watch')
+            show_info += '</span></small>'
+        else:
+            show_info = ''
+        return '<b>%s</b>' % self.name + show_info
+
+    def get_season_info_markup(self, season):
+        if season == '0':
+            name = _('Special')
+        else:
+            name = _('Season %s') % season
+        info = self.get_episodes_info(season)
+        episodes = info['episodes']
+        episodes_to_watch = info['episodes_to_watch']
+        next_episode = info['next_episode']
+        season_info = ''
+        color = get_color(constants.SECONDARY_TEXT_COLOR)
+        if not episodes_to_watch:
+            if episodes:
+                name = '<small><span foreground="%s">%s</span></small>' % \
+                        (get_color(constants.SECONDARY_TEXT_COLOR), name)
+                season_info = _('Completely watched')
+        else:
+            number_episodes_to_watch = len(episodes_to_watch)
+            season_info = gettext.ngettext('%s episode not watched',
+                                           '%s episodes not watched',
+                                           number_episodes_to_watch) \
+                                           % number_episodes_to_watch
+            if next_episode:
+                next_air_date = next_episode.air_date
+                if next_air_date:
+                    season_info += ' | ' + _('<i>Next air date:</i> ep. %s on %s') % \
+                                   (next_episode.episode_number, \
+                                    next_episode.get_air_date_text())
+                else:
+                    season_info += ' | ' + _('<i>Next to watch:</i> %s') % \
+                                   next_episode
+                if next_episode.already_aired():
+                    color = get_color(constants.ACTIVE_TEXT_COLOR)
+        return '<b>%s</b>\n<small><span foreground="%s">%s</span></small>' % \
+               (name, color, season_info)
+
+
 class Episode(object):
     
     def __init__(self, name, show, episode_number, season_number = '1',
@@ -122,8 +202,11 @@ class Episode(object):
         self.watched = watched
         self.air_date = air_date
 
+    def get_episode_show_number(self):
+        return '%sx%02d' % (self.season_number, int(self.episode_number))
+
     def __repr__(self):
-        return _('Ep. %s: %s') % (self.episode_number, self.name)
+        return _('Ep. %s: %s') % (self.get_episode_show_number(), self.name)
     
     def __eq__(self, episode):
         if not episode:
@@ -184,6 +267,13 @@ class Episode(object):
                 return
         self._air_date = None
 
+    def get_info_markup(self):
+        color = get_color(constants.SECONDARY_TEXT_COLOR)
+        if not self.watched and self.already_aired():
+            color = get_color(constants.ACTIVE_TEXT_COLOR)
+        return '<span foreground="%s">%s\n%s</span>' % \
+               (color, self, self.get_air_date_text())
+
     episode_number = property(_get_episode_number, _set_episode_number)
     air_date = property(_get_air_date, _set_air_date)
 
@@ -255,15 +345,19 @@ class SeriesManager(gobject.GObject):
         n_shows = len(show_list)
         for i in range(n_shows):
             show = show_list[i]
+            async_item = AsyncItem(self._set_show_images,
+                                   (show,),
+                                   None,)
+            async_worker.queue.put(async_item)
             async_item = AsyncItem(self.thetvdb.get_show_and_episodes,
                                    (show.thetvdb_id,),
-                                   self._get_show_episodes_complete_cb,
+                                   self._set_show_episodes_complete_cb,
                                    (show, i == n_shows - 1))
             async_worker.queue.put(async_item)
         async_worker.start()
         return async_worker
 
-    def _get_show_episodes_complete_cb(self, show, last_call, tvdbcompleteshow, error):
+    def _set_show_episodes_complete_cb(self, show, last_call, tvdbcompleteshow, error):
         if not error:
             episode_list = [self._convert_thetvdbepisode_to_episode(tvdb_ep,show) \
                             for tvdb_ep in tvdbcompleteshow[1]]
@@ -283,28 +377,31 @@ class SeriesManager(gobject.GObject):
         show_id = self._cached_tvdb_shows.get(show_name, None)
         for show_id, show_title in self._cached_tvdb_shows.items():
             if show_title == show_name:
-                break 
+                break
         if not show_id:
             return
         self.async_worker = AsyncWorker()
-        async_item = AsyncItem(self.thetvdb.get_show_and_episodes,
+        async_item = AsyncItem(self._get_complete_show_from_id,
                                (show_id,),
                                self._get_complete_show_finished_cb)
         self.async_worker.queue.put(async_item)
         self.async_worker.start()
-    
-    def _get_complete_show_finished_cb(self, tvdb_show_episodes, error):
-        show = None
+
+    def _get_complete_show_from_id(self, show_id):
+        tvdb_show_episodes = self.thetvdb.get_show_and_episodes(show_id)
         if not tvdb_show_episodes:
-            return
-        if not error:
-            show = self._convert_thetvdbshow_to_show(tvdb_show_episodes[0])
-            show.id = self._get_id_for_show()
-            show.episode_list = []
-            for tvdb_ep in tvdb_show_episodes[1]:
-                show.episode_list.append(self._convert_thetvdbepisode_to_episode(tvdb_ep,
-                                                                                 show))
-            self.series_list.append(show)
+            return None
+        show = self._convert_thetvdbshow_to_show(tvdb_show_episodes[0])
+        show.id = self._get_id_for_show()
+        show.episode_list = []
+        for tvdb_ep in tvdb_show_episodes[1]:
+            show.episode_list.append(self._convert_thetvdbepisode_to_episode(tvdb_ep,
+                                                                             show))
+        self.series_list.append(show)
+        self._set_show_images(show)
+        return show
+
+    def _get_complete_show_finished_cb(self, show, error):
         self.emit(self.GET_FULL_SHOW_COMPLETE_SIGNAL, show, error)
 
     def get_show_by_id(self, show_id):
@@ -361,6 +458,35 @@ class SeriesManager(gobject.GObject):
                 id += 1
         return id
 
+    def _set_show_images(self, show):
+        thetvdb_id = show.thetvdb_id
+        if thetvdb_id == -1:
+            return
+        image_choices = self.thetvdb.get_show_image_choices(thetvdb_id)
+        seasons = show.get_seasons()
+        for key, image in show.season_images.items():
+            if not os.path.isfile(image):
+                del show.season_images[key]
+        for image in image_choices:
+            image_type = image[1]
+            url = image[0]
+            if image_type  == 'poster' and \
+               (not show.image or not os.path.isfile(show.image)):
+                target_file = os.path.join(DATA_DIR, show.thetvdb_id)
+                image_file = os.path.abspath(image_downloader(url, target_file))
+                show.image = image_file
+            elif image_type == 'season':
+                season = image[3]
+                if season in seasons and \
+                   season not in show.season_images.keys():
+                    target_file = os.path.join(DATA_DIR,
+                                          show.thetvdb_id + '_season_' + season)
+                    image_file = os.path.abspath(image_downloader(url,
+                                                                  target_file))
+                    show.season_images[season] = image_file
+            if show.image and len(show.season_images) == len(seasons):
+                break
+
     def save(self, save_file_path):
         dirname = os.path.dirname(save_file_path)
         if not (os.path.exists(dirname) and os.path.isdir(dirname)):
